@@ -306,7 +306,14 @@ def get_parameter_data_for_one_paramtree(
     # faster than transposing the data using np.array.transpose
     res_t = map(list, zip(*data))
 
-    for paramspec, column_data in zip_longest(paramspecs, res_t, fillvalue=tuple()):
+    specs_and_data: zip_longest[
+        Tuple[Union[ParamSpecBase, Sequence[Any]], Sequence[Any]]
+    ] = zip_longest(paramspecs, res_t, fillvalue=())
+
+    for paramspec, column_data in specs_and_data:
+        assert isinstance(paramspec, ParamSpecBase)
+        # its not obvious how to type that paramspecs is always
+        # longer than res_t
         if paramspec.type == "numeric":
             # there is no reliable way to
             # tell the difference between a float and and int loaded
@@ -833,7 +840,12 @@ def new_experiment(conn: ConnectionPlus,
     start_time = start_time or time.time()
     values = (name, sample_name, format_string, 0, start_time, end_time)
     curr = atomic_transaction(conn, query, *values)
-    return curr.lastrowid
+
+    return_value = curr.lastrowid
+
+    if return_value is None:
+        raise RuntimeError(f"Insert of new experiment with {name} failed")
+    return return_value
 
 
 # TODO(WilliamHPNielsen): we should remove the redundant
@@ -1277,6 +1289,8 @@ def _insert_run(
                 parent_dataset_links,
             )
             run_id = curr.lastrowid
+            if run_id is None:
+                raise RuntimeError(f"Creation of run with guid: {guid} failed")
 
             _add_parameters_to_layout_and_deps(conn, run_id, *legacy_param_specs)
 
@@ -1311,7 +1325,8 @@ def _insert_run(
                                parent_dataset_links)
 
     run_id = curr.lastrowid
-
+    if run_id is None:
+        raise RuntimeError(f"Creation of run with guid: {guid} failed")
     return run_counter, formatted_name, run_id
 
 
