@@ -1,7 +1,7 @@
-import warnings
 import types
+import warnings
 from contextlib import contextmanager
-from typing import Optional, Callable, Any, cast, Iterator, List
+from typing import Any, Callable, Iterator, List, Optional, cast
 
 import wrapt
 
@@ -28,7 +28,7 @@ def issue_deprecation_warning(
     what: str,
     reason: Optional[str] = None,
     alternative: Optional[str] = None,
-    stacklevel: int = 2
+    stacklevel: int = 2,
 ) -> None:
     warnings.warn(
         deprecation_message(what, reason, alternative),
@@ -51,12 +51,15 @@ def deprecate(
     """
 
     @wrapt.decorator  # type: ignore[misc]
-    def decorate_callable(func: Callable[..., Any],
-                          instance: object, args: Any, kwargs: Any) -> Any:
-        t, n = (('class', instance.__class__.__name__)
-                if func.__name__ == '__init__'
-                else ('function', func.__name__))
-        issue_deprecation_warning(f'{t} <{n}>', reason, alternative)
+    def decorate_callable(
+        func: Callable[..., Any], instance: object, args: Any, kwargs: Any
+    ) -> Any:
+        t, n = (
+            ("class", instance.__class__.__name__)
+            if func.__name__ == "__init__"
+            else ("function", func.__name__)
+        )
+        issue_deprecation_warning(f"{t} <{n}>", reason, alternative, stacklevel=3)
         return func(*args, **kwargs)
 
     def actual_decorator(obj: Any) -> Any:
